﻿using System;
using System.Net;
using System.Threading;
using AdGuard.Utils.Http;
// ReSharper disable LocalizableElement

namespace Adguard.Core.Tests.Helpers
{
    /// <summary>
    ///   This proxy implementation is a workaround for WebRequest.DefaultWebProxy being very slow 
    ///   (first request to WebRequest.DefaultWebProxy takes 10-30 seconds).
    ///   This proxy checks registry proxy settings and works in following ways:
    ///   1. If no proxy set -- initializes WebRequest.DefaultWebProxy asynchronously, using empty proxy instead.
    ///   2. If proxy is set -- uses WebRequest.DefaultWebProxy (this is rather rare situation, so be it).
    /// </summary>
    public class DefaultWebProxy : IWebProxy
    {
        private readonly object m_SyncRoot = new object();
        private IWebProxy m_BaseProxy;

        /// <summary>
        ///   Creates an instance of the DefaultWebProxy.
        ///   Starts WebRequest.DefaultWebProxy async initializing.
        /// </summary>
        public DefaultWebProxy()
        {
            if (HttpUtils.IsProxyEnabled)
            {
                // Proxy is enabled, doing nothing
                BaseProxy = WebRequest.DefaultWebProxy;
            }
            else
            {
                // Using empty proxy while initializing
                BaseProxy = new WebProxy();
                // Asynchronously initializing DefaultWebProxy
                ThreadPool.QueueUserWorkItem(obj => InitializeDefaultProxy());
            }
        }

        /// <summary>
        ///   Synchronized propery to access base proxy
        /// </summary>
        private IWebProxy BaseProxy
        {
            get
            {
                lock (m_SyncRoot)
                {
                    return m_BaseProxy;
                }
            }
            set
            {
                lock (m_SyncRoot)
                {
                    m_BaseProxy = value;
                }
            }
        }

        /// <summary>
        ///   Initializes WebRequest.DefaultWebProxy (just does a GetProxy request)
        /// </summary>
        private void InitializeDefaultProxy()
        {
            try
            {
                Console.WriteLine("Initializing default proxy");
                var testUri = new Uri("http://api.adguard.com/");
                Uri proxyUrl = WebRequest.DefaultWebProxy.GetProxy(testUri);
                Console.WriteLine("Default proxy has been initialized.");
                if (testUri != proxyUrl)
                {
                    Console.WriteLine("Requests are proxified to {0}", proxyUrl);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Cannot initialize default proxy: {0}", ex);
            }
            finally
            {
                BaseProxy = WebRequest.DefaultWebProxy;
            }
        }

        /// <summary>
        /// Gets proxy url for the specified destination
        /// </summary>
        /// <param name="destination">Destination uri</param>
        /// <returns>Proxy uri</returns>
        public Uri GetProxy(Uri destination)
        {
            return BaseProxy.GetProxy(destination);
        }

        /// <summary>
        /// Checks if specified uri is bypassed by this proxy
        /// </summary>
        /// <param name="host">Host to check</param>
        /// <returns>true if bypassed</returns>
        public bool IsBypassed(Uri host)
        {
            return BaseProxy.IsBypassed(host);
        }

        /// <summary>
        /// Gets or sets proxy credentials
        /// </summary>
        public ICredentials Credentials
        {
            get { return BaseProxy.Credentials; }
            set { BaseProxy.Credentials = value; }
        }

        public override string ToString()
        {
            return "DefaultWebProxy";
        }
    }
}