﻿using System;
using System.Collections.Generic;
using Adguard.Core.AGApi;
using Adguard.Core.Api.Certificate;
using Adguard.Core.Utils;
using NUnit.Framework;

namespace Adguard.Core.Tests.Helpers
{
    [TestFixture]
    public class TestProxyUtils
    {
        [Test]
        public void TestGenerateCACertificate()
        {
            Assert.Throws<ArgumentException>(() => ProxyUtils.GenerateCACertificate(string.Empty));
            CACertificate cert = ProxyUtils.GenerateCACertificate("test certificate");
            Assert.AreNotEqual(cert.RootCertificate, 0);
            Assert.AreNotEqual(cert.RootCertificateKey, 0);
            Assert.NotNull(cert.Pem);
            Assert.AreNotEqual(cert.Pem, string.Empty);
        }

        [Test]
        public void TestValidateCACertificate()
        {
            CACertificate cert = ProxyUtils.GenerateCACertificate("test certificate");
            Assert.AreNotEqual(cert.RootCertificate, 0);
            Assert.AreNotEqual(cert.RootCertificateKey, 0);
            Assert.NotNull(cert.Pem);
            Assert.AreNotEqual(cert.Pem, string.Empty);
            AGWinProxyUtilsApi.AGPUCAValidationStatus status = ProxyUtils.ValidateCACertAndKey(cert.RootCertificate, cert.RootCertificateKey);
            Assert.AreEqual(status, AGWinProxyUtilsApi.AGPUCAValidationStatus.AGCAVS_OK);
            byte firstCertByte = cert.RootCertificate[0];
            cert.RootCertificate[0] = byte.MaxValue;
            status = ProxyUtils.ValidateCACertAndKey(cert.RootCertificate, cert.RootCertificateKey);
            Assert.AreEqual(status, AGWinProxyUtilsApi.AGPUCAValidationStatus.AGCAVS_INVALID_CERT);
            cert.RootCertificate[0] = firstCertByte;
            cert.RootCertificateKey[0] = byte.MaxValue;
            status = ProxyUtils.ValidateCACertAndKey(cert.RootCertificate, cert.RootCertificateKey);
            Assert.AreEqual(status, AGWinProxyUtilsApi.AGPUCAValidationStatus.AGCAVS_INVALID_KEY);
            CACertificate cert1 = ProxyUtils.GenerateCACertificate("test certificate");
            status = ProxyUtils.ValidateCACertAndKey(cert.RootCertificate, cert1.RootCertificateKey);
            Assert.AreEqual(status, AGWinProxyUtilsApi.AGPUCAValidationStatus.AGCAVS_KEY_DOES_NOT_MATCH_CERT);
        }

        [Test]
        public void TestReadCACertificate()
        {
            Assert.Throws<ArgumentException>(() => ProxyUtils.ReadCACertificate(string.Empty, string.Empty));
            CACertificate cert1 = ProxyUtils.GenerateCACertificate("test certificate");
            Assert.NotNull(cert1.Pem);
            Assert.AreNotEqual(cert1.Pem, string.Empty);

            CACertificate cert2 = ProxyUtils.ReadCACertificate(cert1.Pem, string.Empty);
            Assert.AreNotEqual(cert2.RootCertificate, 0);
            Assert.AreNotEqual(cert2.RootCertificateKey, 0);
            Assert.NotNull(cert2.Pem);
            Assert.AreNotEqual(cert2.Pem, string.Empty);
        }

        [Test]
        public void TesRuleValidation()
        {
            Dictionary<string, bool> ruleValidationTable = new Dictionary<string, bool>
            {
                {"##.ad", true},
                {"#@#.ad", true},
                {"#?#.ad", true},
                {"##.ad:style(background-color:red)", true},
                {"#$#body {background-color:red}", true},
                {"#$##adblock {background-color:red}", true},
                {"||blabla.com", true},
                {"@@||blabla.com", true},
                {"||blabla.com$replace=/foo/bar/", true},
                {"||blabla.com$rreplace=/foo/bar/", false},
                {"example.org$$script[data-src=\"banner\"]", true},
                {"||example.org^$third-party", true},
                {"||example.org", true},
                {"||example.org/example.html", true},
                {"@@||example.org^$content", true},
                {"@@||example.org^$cookie,content", false},
                {"@@||example.org^$cookie=/__utm[a-z]/,important", true},
                {"@@||example.org^$csp,important", true},
                {"@@||example.org/example.html", true}
            };

            foreach (var pair in ruleValidationTable)
            {
                Console.WriteLine("Rule: {0}", pair.Key);
                Assert.AreEqual(ProxyUtils.IsRuleValid(pair.Key), pair.Value);
            }
        }

        [Test]
        public void TestGetTld1Domain()
        {
            string tld1 = "foo.bas.it";
            string[] hosts =
            {
                "aaa.foo.bas.it",
                "bbb.aaa.foo.bas.it"
            };

            foreach (string host in hosts)
            {
                string tld1Calc = ProxyUtils.GetTld1Domain(host);
                Console.WriteLine("host: {0}", host);
                Assert.AreEqual(tld1, tld1Calc);
            }
        }

        [Test]
        public void TestGetWhitelistRuleDomain()
        {
            string ruleText = "@@||yandex.ru^$important,document";
            string whitelistRuleDomain = "yandex.ru";
            string calcWhitelistRuleDomain = ProxyUtils.GetWhitelistRuleDomain(ruleText);
            Console.WriteLine("whitelistRuleDomain: {0}", calcWhitelistRuleDomain);
            Assert.AreEqual(whitelistRuleDomain, calcWhitelistRuleDomain);
            string calcRuleText = ProxyUtils.BasicWhitelistRuleWithDomain(whitelistRuleDomain);
            Console.WriteLine("calcRuleText: {0}", calcRuleText);
            Assert.AreEqual(ruleText, calcRuleText);
        }

        [Test]
        public void TestInvalidGetTld1Domain()
        {
            string[] invalidHosts =
            {
                "aaafoobasit",
                "aaafoobasit.",
                null,
                string.Empty
            };

            foreach (string invalidHost in invalidHosts)
            {
                Console.WriteLine("invalid host: {0}", invalidHost);
                string tld1Calc = ProxyUtils.GetTld1Domain(invalidHost);
                Assert.IsTrue(string.IsNullOrEmpty(tld1Calc));
            }
        }
    }
}