﻿using System;
using System.IO;
using System.Linq;
using Adguard.Core.AGApi;
using Adguard.Core.Api.ProxyServer.Configs;
using Adguard.Core.Common.Logging;
using AdGuard.Utils.Logging;
using Adguard.Core.Userscripts;
using NUnit.Framework;

namespace Adguard.Core.Tests.TestApi.GmStorage
{
    [TestFixture]
    public class TestGmStorage
    {
        private string m_Userscript1Name = "Adguard Assistant Dev";
        private string m_Userscript2Name = "Adguard Assistant Beta";

        [Test]
        public void TestCoreLibsInit()
        {
            Assert.DoesNotThrow(() =>
            {
                CoreLibsInitializer coreLibsInitializer = CoreLibsInitializer.New();
                coreLibsInitializer.Dispose();
            });
        }

        [Test]
        public void TestSetGmValue()
        {
            using (CoreLibsInitializer.New())
            {
                Assert.DoesNotThrow(() =>
                {
                    GMStorage.SetUserscriptGmProperty(m_Userscript1Name, "aaa", "bbb");
                });
            }
        }

        [Test]
        public void TestGetGmValue()
        {
            using (CoreLibsInitializer.New())
            {
                string gmPropName = "gmPropName";
                string gmPropValue = "gmPropValue";
                GMStorage.SetUserscriptGmProperty(m_Userscript1Name, gmPropName, gmPropValue);
                string property = GMStorage.GetUserscriptGmProperty(m_Userscript1Name, gmPropName);
                Assert.AreEqual(property, gmPropValue);
            }
        }

        [Test]
        public void TestGetNotExistingGmValue()
        {
            using (CoreLibsInitializer.New())
            {
                string gmPropName = "gmPropName";
                string property = GMStorage.GetUserscriptGmProperty(m_Userscript1Name, gmPropName);
                Assert.IsNull(property);
            }
        }

        [Test]
        public void TestDeleteAllGmValues()
        {
            using (CoreLibsInitializer.New())
            {
                string gmPropName1 = "gmPropName1";
                string gmPropValue1 = "gmPropValue1";

                string gmPropName2 = "gmPropName2";
                string gmPropValue2 = "gmPropValue2";

                GMStorage.SetUserscriptGmProperty(m_Userscript1Name, gmPropName1, gmPropValue1);
                GMStorage.SetUserscriptGmProperty(m_Userscript1Name, gmPropName2, gmPropValue2);

                var properties = GMStorage.GetGmProperties(m_Userscript1Name);
                Assert.AreEqual(properties.Count, 2);

                GMStorage.DeleteAllUserscriptGmProperties(m_Userscript1Name);

                string property = GMStorage.GetUserscriptGmProperty(m_Userscript1Name, gmPropName1);
                Assert.IsNull(property);

                property = GMStorage.GetUserscriptGmProperty(m_Userscript1Name, gmPropName2);
                Assert.IsNull(property);

                properties = GMStorage.GetGmProperties(m_Userscript1Name);
                Assert.AreEqual(properties.Count, 0);
            }
        }

        [Test]
        public void TestDeleteSingleGmValue()
        {
            using (CoreLibsInitializer.New())
            {
                string gmPropName1 = "gmPropName1";
                string gmPropValue1 = "gmPropValue1";

                string gmPropName2 = "gmPropName2";
                string gmPropValue2 = "gmPropValue2";

                GMStorage.SetUserscriptGmProperty(m_Userscript1Name, gmPropName1, gmPropValue1);
                GMStorage.SetUserscriptGmProperty(m_Userscript1Name, gmPropName2, gmPropValue2);

                var properties = GMStorage.GetGmProperties(m_Userscript1Name);
                Assert.AreEqual(properties.Count, 2);

                GMStorage.DeleteUserscriptGmProperty(m_Userscript1Name, gmPropName1);

                string property = GMStorage.GetUserscriptGmProperty(m_Userscript1Name, gmPropName1);
                Assert.IsNull(property);

                property = GMStorage.GetUserscriptGmProperty(m_Userscript1Name, gmPropName2);
                Assert.IsNotNull(property);
                Assert.AreEqual(property, gmPropValue2);

                properties = GMStorage.GetGmProperties(m_Userscript1Name);
                Assert.AreEqual(properties.Count, 1);
            }
        }

        [Test]
        public void TestDeleteNonExistingSingleGmValue()
        {
            using (CoreLibsInitializer.New())
            {
                string gmPropName1 = "gmPropName1";
                string gmPropValue = GMStorage.GetUserscriptGmProperty(m_Userscript1Name, gmPropName1);
                Assert.Null(gmPropValue);
                Assert.DoesNotThrow(() =>
                {

                    GMStorage.DeleteUserscriptGmProperty(m_Userscript1Name, gmPropName1);
                });
            }
        }

        [Test]
        public void TestDeleteAllNonExistingGmValues()
        {
            using (CoreLibsInitializer.New())
            {
                var properties = GMStorage.GetGmProperties(m_Userscript1Name);
                Assert.AreEqual(properties.Count, 0);
                Assert.DoesNotThrow(() =>
                {
                    GMStorage.DeleteAllUserscriptGmProperties(m_Userscript1Name);
                });
            }
        }

        [Test]
        public void TestGetUserscriptsList()
        {
            using (CoreLibsInitializer.New())
            {
                string gmPropName1 = "gmPropName1";
                string gmPropValue1 = "gmPropValue1";

                string gmPropName2 = "gmPropName2";
                string gmPropValue2 = "gmPropValue2";

                GMStorage.SetUserscriptGmProperty(m_Userscript1Name, gmPropName1, gmPropValue1);
                GMStorage.SetUserscriptGmProperty(m_Userscript2Name, gmPropName2, gmPropValue2);

                var userscripts = GMStorage.GetUserscripts();
                Assert.IsNotNull(userscripts);
                Assert.AreEqual(userscripts.Count, 2);
                Assert.True(userscripts.Contains(m_Userscript1Name));
                Assert.True(userscripts.Contains(m_Userscript2Name));
            }
        }

        [Test]
        public void TestGetAllGmProperties()
        {
            using (CoreLibsInitializer.New())
            {
                string gmPropName1 = "gmPropName1";
                string gmPropValue1 = "gmPropValue1";

                string gmPropName2 = "gmPropName2";
                string gmPropValue2 = "gmPropValue2";

                GMStorage.SetUserscriptGmProperty(m_Userscript1Name, gmPropName1, gmPropValue1);
                GMStorage.SetUserscriptGmProperty(m_Userscript1Name, gmPropName2, gmPropValue2);

                var properties = GMStorage.GetGmProperties(m_Userscript1Name);
                Assert.AreEqual(properties.Count, 2);
                Assert.True(properties.Keys.Contains(gmPropName1));
                Assert.True(properties.Keys.Contains(gmPropName2));
                Assert.True(properties.Values.Contains(gmPropValue1));
                Assert.True(properties.Values.Contains(gmPropValue2));
            }
        }
    }

    internal class CoreLibsInitializer : IDisposable
    {
        private readonly string m_CoreLibsDirectory;

        private CoreLibsInitializer()
        {
            CoreConfiguration coreConfiguration = TestConfigurationHelper.GetNewCoreConfiguration();
            m_CoreLibsDirectory =  coreConfiguration.CorelibsDirectory;
            Directory.CreateDirectory(m_CoreLibsDirectory);
            CoreLibs.Init(coreConfiguration);
        }

        public static CoreLibsInitializer New()
        {
            CoreLoggerAdapter.Init(AGLoggerApi.logger_log_level_t.LOG_LEVEL_TRACE);
            CoreLoggerAdapter.SetLogger(AGCApi.CoreLibName);
            return new CoreLibsInitializer();
        }

        public void Dispose()
        {
            CoreLibs.DeInit();
            Directory.Delete(m_CoreLibsDirectory, true);
        }
    }
}